' Note: To run this inside Visual Studio, you will need to:
' Project / Properties / Debug / Uncheck Enable the Visual Studio hosting process

' Usage 
' Dim Mouse_OnOff As New MouseOnOff()
' Mouse_OnOff.SetMouseHook()
' Mouse_OnOff.ReleaseMouseHook()

Imports System.Runtime.InteropServices
Imports System.Reflection.Assembly

Public Class MouseOnOff

    Private MouseHook As Integer

    Private Const HC_ACTION As Integer = 0
    Private Const WH_MOUSE_LL As Integer = 14
    Private Const WM_MOUSEMOVE As Integer = &H200
    Private Const WM_LBUTTONDOWN As Integer = &H201
    Private Const WM_LBUTTONUP As Integer = &H202
    Private Const WM_LBUTTONDBLCLK As Integer = &H203
    Private Const WM_RBUTTONDOWN As Integer = &H204
    Private Const WM_RBUTTONUP As Integer = &H205
    Private Const WM_RBUTTONDBLCLK As Integer = &H206
    Private Const WM_MBUTTONDOWN As Integer = &H207
    Private Const WM_MBUTTONUP As Integer = &H208
    Private Const WM_MBUTTONDBLCLK As Integer = &H209
    Private Const WM_MOUSEWHEEL As Integer = &H20A
    Private Const WM_XBUTTONDOWN As Integer = &H20B
    Private Const WM_XBUTTONUP As Integer = &H20C
    Private Const WM_XBUTTONDBLCLK As Integer = &H20D

    Private Structure MSLLHOOKSTRUCT
        Public pt As Point
        Public mouseData As Integer
        Public flags As Integer
        Public time As Integer
        Public dwExtraInfo As Integer
    End Structure

    Private Declare Function SetWindowsHookEx Lib "user32" _
                                            Alias "SetWindowsHookExA" ( _
                                            ByVal idHook As Integer, _
                                            ByVal lpfn As LowLevelMouseProcDelegate, _
                                            ByVal hmod As Integer, _
                                            ByVal dwThreadId As Integer) As Integer

    Private Declare Function CallNextHookEx Lib "user32" ( _
                                            ByVal hHook As Integer, _
                                            ByVal nCode As Integer, _
                                            ByVal wParam As Integer, _
                                            ByVal lParam As MSLLHOOKSTRUCT) As Integer

    Private Declare Function UnhookWindowsHookEx Lib "user32" ( _
                                            ByVal hHook As Integer) As Integer

    Private MouseHookDelegate As LowLevelMouseProcDelegate

    Private Delegate Function LowLevelMouseProcDelegate( _
                                            ByVal nCode As Integer, _
                                            ByVal wParam As Integer, _
                                            ByRef lParam As MSLLHOOKSTRUCT) As Integer

    Private Function LowLevelMouseProc(ByVal nCode As Integer, _
                                       ByVal wParam As Integer, _
                                       ByRef lParam As MSLLHOOKSTRUCT) As Integer

        If (nCode = HC_ACTION) Then

            If wParam >= WM_MOUSEMOVE And _
               wParam <= WM_XBUTTONDBLCLK Then

                If MouseEnabled Then
                    DisableCounter = 0
                Else
                    If wParam = WM_LBUTTONUP Then
                        TestMousePosition(lParam.pt)
                        TestMouseSequence("L")
                    End If
                    If wParam = WM_RBUTTONUP Then
                        TestMouseSequence("R")
                    End If
                    If wParam <> WM_MOUSEMOVE Then
                        Return 1
                    End If
                End If
            End If
        End If
        Return CallNextHookEx(MouseHook, nCode, wParam, lParam)
    End Function

    Friend MouseSequence As String = "LRL"
    Private SequenceIndex As Int32 = 1
    Private WithEvents timer1 As Timer = New Timer
    Private Sub TestMouseSequence(ByVal code As String)
        If MouseSequence.Length = 0 Then Return
        If code = Strings.Mid(MouseSequence.ToUpper, SequenceIndex, 1) Then
            SequenceIndex += 1
        Else
            SequenceIndex = 1
        End If
        If SequenceIndex > MouseSequence.Length Then
            SequenceIndex = 1
            DisableCounter = -1
        Else
            timer1.Interval = 1000
            timer1.Stop()
            timer1.Start()
        End If
    End Sub

    Private Sub timer1_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles timer1.Tick
        timer1.Stop()
        SequenceIndex = 1
    End Sub

    Friend EnablePoint As Point
    Friend EnablePointRadius As Single
    Private Sub TestMousePosition(ByVal pt As Point)
        Debug.Print(pt.X.ToString + "   " + pt.Y.ToString)
        If PointDistance(pt, EnablePoint) < EnablePointRadius Then
            SequenceIndex = 1
            DisableCounter = -1
            'Beep()
        End If
    End Sub

    Friend Sub SetMouseHook()
        MouseHookDelegate = New LowLevelMouseProcDelegate(AddressOf LowLevelMouseProc)
        MouseHook = SetWindowsHookEx(WH_MOUSE_LL, _
                                     MouseHookDelegate, _
                                     Marshal.GetHINSTANCE( _
                                     GetExecutingAssembly.GetModules()(0)).ToInt32, _
                                     0)
    End Sub

    Friend Sub ReleaseMouseHook()
        UnhookWindowsHookEx(MouseHook)
    End Sub

End Class



